/*------------------------------------------------------------------------------
	Newton interface to the MD5 Message Digest algorithm.
	Input:
		Unicode string; < 256 characters to be digested
	Output:
		Unicode string; 32 hex characters
------------------------------------------------------------------------------*/

#include "Objects.h"
#include "Unicode.h"
#include "UStringUtils.h"
#include "MD5.h"

#ifdef __cplusplus
#define EXTERNC extern "C"
#else
#define EXTERNC
#endif

EXTERNC Ref Digest(RefArg rcvr, RefArg string)
{
	RefVar result = NILREF;

	//	parameter must be a string
	if (IsString(string))
	{
		MD5_context context;
		unsigned char digest[16];
		unsigned int len;
		char cstring[256];

		//	get length of string and convert Unicode -> ASCII
		WITH_LOCKED_BINARY(string, s)
		len = Ustrlen((UniChar*) s);
		ConvertFromUnicode((const UniChar*) s, cstring, kDefaultEncoding, 255);
		cstring[255] = '\0';
		END_WITH_LOCKED_BINARY(string)

		//	digest
		MD5Init (&context);
		MD5Update (&context, (unsigned char*)cstring, len);
		MD5Final (digest, &context); 

		//	convert to hex characters
		const char hexDigits[] = "0123456789abcdef";
		char *cp = cstring;
		for (len = 0; len < 16; len++)
		{
			char c = digest[len];
			*cp++ = hexDigits[c >> 4];
			*cp++ = hexDigits[c & 0x0F];
		}
		*cp = '\0';

		//	return Unicode string
		result = MakeString((const char *)cstring);
	}

	return result;
}
